#==================================
#========Reward Gradient TI========
#==================================

#====================
#====Front Matter====
#====================

library(rstan)
library(rethinking)
library(readr)

RGMD1 <- read_csv("~/Projects/Academia/0 - In Preparation/**The Vince Ferrera Back Catalog/Reward Gradient TI/RGMD-1.csv")
RGMD2 <- read_csv("~/Projects/Academia/0 - In Preparation/**The Vince Ferrera Back Catalog/Reward Gradient TI/RGMD-2.csv")

#=================================
#====Stan Model of Performance====
#=================================
# These Stan models are responsible for fitting binomial regressions to session performance,
# using a logit link. In each case, the population-level parameters are denoted by "gamma" and
# the subject-level parameters are denoted by "beta." For example, gamma[1] is the population
# intercept term, whereas beta[3,1] is the intercept term for Subject 3. The relevant parameters
# for each model are as follows:
# * all_pair_model_string (Experiment 1 only)
#   - gamma[1]  -  Intercept
#   - gamma[2]  -  Symbolic Distance
# * reward_gradient_model_string (Experiments 1 and 2)
#   - gamma[1]  -  Intercept
#   - gamma[2]  -  Symbolic Distance
#   - gamma[3]  -  Reward Magnitude
# * reward_gradient_adjacent_model_string (Experiment 2 only)
#   - gamma[1]  -  Intercept
#   - gamma[2]  -  Reward Magnitude
# * marginal_pseudobetasort_machine (Experiment 2 only)
#   - gamma     -  Estimated Stimulus Positions
#   - bigsigma  -  Estimated Stimulus Variation
#   - bigbeta1  -  Shape1 term for population beta distribution of random behaviors
#   - bigbeta2  -  Shape2 term for population beta distribution of random behaviors
# * q_learning_machine (Experiment 2 only)
#   - alpha     -  Learning Rate Parameter
#   - beta      -  Exponential Softmax Value Parameter

all_pair_machine <- stan_model(file="all_pair_model.stan")
reward_gradient_machine <- stan_model(file="reward_gradient_model.stan")
reward_gradient_adjacent_machine <- stan_model(file="reward_gradient_adjacent_model.stan")
marginal_pseudobetasort_machine <- stan_model(file="marginal_pseudobetasort_model.stan")
q_learning_machine <- stan_model(file="q_learning_one_subject_model.stan")

machines <- list()
machines[1] <- list(all_pair_machine)
machines[2] <- list(reward_gradient_machine)
machines[3] <- list(reward_gradient_adjacent_machine)
machines[4] <- list(marginal_pseudobetasort_machine)
machines[5] <- list(q_learning_machine)


#===============================================================
#====Experiment 1 - 4 Drops vs. Reverse Gradient - All Pairs====
#===============================================================
# Performance is modeled on a session by session basis (under the "Session Performance" headers)
# and the resulting model parameters are then averaged (under the "Mean Performance" headers).

#====Phase 1 Session Performance====

exp1_ph1_all_pair_output <- list()
for (i in 1:5) {
  dex <- (RGMD1$Phase==1)&(RGMD1$Session==i)
  dat1 <- list(N=sum(dex), S=length(unique(RGMD1$Subject[dex])), sub=RGMD1$Subject[dex], dist=RGMD1$Distance[dex]-3.5, success=RGMD1$Successes[dex], attempt=RGMD1$Attempts[dex])
  exp1_ph1_all_pair_output[i] <- list(sampling(all_pair_machine, data=dat1, iter=4000, warmup=2000, chains=4, cores=4, control=list(adapt_delta=0.95)))
  precis(exp1_ph1_all_pair_output[[i]],pars="gamma",depth=2)
}

#====Phase 2 Session Performance====

exp1_ph2_reward_gradient_output <- list()
for (i in 1:11) {
  dex <- (RGMD1$Phase==2)&(RGMD1$Session==i)
  dat <- list(N=sum(dex), S=length(unique(RGMD1$Subject[dex])), sub=RGMD1$Subject[dex], dist=RGMD1$Distance[dex]-3.5, reward=RGMD1$Reward[dex]-3.5, success=RGMD1$Successes[dex], attempt=RGMD1$Attempts[dex])
  exp1_ph2_reward_gradient_output[i] <- list(sampling(reward_gradient_machine, data=dat, iter=4000, warmup=2000, chains=4, cores=4, control=list(adapt_delta=0.95)))
  precis(exp1_ph2_reward_gradient_output[[i]],pars="gamma",depth=2)
}

#====Phase 3 Session Performance====

exp1_ph3_all_pair_output <- list()
for (i in 1:9) {
  dex <- (RGMD1$Phase==3)&(RGMD1$Session==i)
  dat <- list(N=sum(dex), S=length(unique(RGMD1$Subject[dex])), sub=RGMD1$Subject[dex], dist=RGMD1$Distance[dex]-3.5, success=RGMD1$Successes[dex], attempt=RGMD1$Attempts[dex])
  exp1_ph3_all_pair_output[i] <- list(sampling(all_pair_machine, data=dat, iter=4000, warmup=2000, chains=4, cores=4, control=list(adapt_delta=0.95)))
  precis(exp1_ph3_all_pair_output[[i]],pars="gamma",depth=2)
}

#====Phase 4 Session Performance====

exp1_ph4_reward_gradient_output <- list()
for (i in 1:9) {
  dex <- (RGMD1$Phase==4)&(RGMD1$Session==i)
  dat <- list(N=sum(dex), S=length(unique(RGMD1$Subject[dex])), sub=RGMD1$Subject[dex], dist=RGMD1$Distance[dex]-3.5, reward=RGMD1$Reward[dex]-3.5, success=RGMD1$Successes[dex], attempt=RGMD1$Attempts[dex])
  exp1_ph4_reward_gradient_output[i] <- list(sampling(reward_gradient_machine, data=dat, iter=4000, warmup=2000, chains=4, cores=4, control=list(adapt_delta=0.95)))
  precis(exp1_ph4_reward_gradient_output[[i]],pars="gamma",depth=2)
}

#====Phase 1 Mean Performance====

sess <- 5
exp1_ph1_qmean <- extract.samples(exp1_ph1_all_pair_output[[1]])
exp1_ph1_qmean$z <- NULL
exp1_ph1_qmean$L_Omega <- NULL
exp1_ph1_qmean$tau_unif <- NULL
exp1_ph1_qmean$gamma <- exp1_ph1_qmean$gamma/sess
exp1_ph1_qmean$beta <- exp1_ph1_qmean$beta/sess
exp1_ph1_qmean$tau <- NULL
exp1_ph1_qmean$lp__ <- NULL
for (i in 2:sess) {
	q <- extract.samples(exp1_ph1_all_pair_output[[i]])
	exp1_ph1_qmean$gamma <- exp1_ph1_qmean$gamma + (q$gamma/sess)
	exp1_ph1_qmean$beta <- exp1_ph1_qmean$beta + (q$beta/sess)
}

#====Phase 2 Mean Performance====

sess <- 11
exp1_ph2_qmean <- extract.samples(exp1_ph2_reward_gradient_output[[1]])
exp1_ph2_qmean$z <- NULL
exp1_ph2_qmean$L_Omega <- NULL
exp1_ph2_qmean$tau_unif <- NULL
exp1_ph2_qmean$gamma <- exp1_ph2_qmean$gamma/sess
exp1_ph2_qmean$beta <- exp1_ph2_qmean$beta/sess
exp1_ph2_qmean$tau <- NULL
exp1_ph2_qmean$lp__ <- NULL
for (i in 2:sess) {
	q <- extract.samples(exp1_ph2_reward_gradient_output[[i]])
	exp1_ph2_qmean$gamma <- exp1_ph2_qmean$gamma + (q$gamma/sess)
	exp1_ph2_qmean$beta <- exp1_ph2_qmean$beta + (q$beta/sess)
}

#====Phase 3 Mean Performance====

sess <- 9
exp1_ph3_qmean <- extract.samples(exp1_ph3_all_pair_output[[1]])
exp1_ph3_qmean$z <- NULL
exp1_ph3_qmean$L_Omega <- NULL
exp1_ph3_qmean$tau_unif <- NULL
exp1_ph3_qmean$gamma <- exp1_ph3_qmean$gamma/sess
exp1_ph3_qmean$beta <- exp1_ph3_qmean$beta/sess
exp1_ph3_qmean$tau <- NULL
exp1_ph3_qmean$lp__ <- NULL
for (i in 2:sess) {
	q <- extract.samples(exp1_ph3_all_pair_output[[i]])
	exp1_ph3_qmean$gamma <- exp1_ph3_qmean$gamma + (q$gamma/sess)
	exp1_ph3_qmean$beta <- exp1_ph3_qmean$beta + (q$beta/sess)
}

#====Phase 4 Mean Performance====

sess <- 9
exp1_ph4_qmean <- extract.samples(exp1_ph4_reward_gradient_output[[1]])
exp1_ph4_qmean$z <- NULL
exp1_ph4_qmean$L_Omega <- NULL
exp1_ph4_qmean$tau_unif <- NULL
exp1_ph4_qmean$gamma <- exp1_ph4_qmean$gamma/sess
exp1_ph4_qmean$beta <- exp1_ph4_qmean$beta/sess
exp1_ph4_qmean$tau <- NULL
exp1_ph4_qmean$lp__ <- NULL
for (i in 2:sess) {
	q <- extract.samples(exp1_ph4_reward_gradient_output[[i]])
	exp1_ph4_qmean$gamma <- exp1_ph4_qmean$gamma + (q$gamma/sess)
	exp1_ph4_qmean$beta <- exp1_ph4_qmean$beta + (q$beta/sess)
}

#====Output====
# Results are then exported to the clipboard for figure construction using the write.table() function.

outputs <- matrix(0,nrow=34,ncol=15)
for (i in 1:5) {
  q <- extract.samples(ph1_all_pair_output[[i]])
  v <- inv_logit(q$gamma[,1] + q$gamma[,2]*(-2.5)*(6/21) + q$gamma[,2]*(-1.5)*(5/21) + q$gamma[,2]*(-0.5)*(4/21) + q$gamma[,2]*(0.5)*(3/21) + q$gamma[,2]*(1.5)*(2/21) + q$gamma[,2]*(2.5)*(1/21))
  outputs[i,1] <- mean(v)
  outputs[i,2:5] <- quantile(v,c(.025,.975,.1,.9))
  outputs[i,6] <- mean(q$gamma[,2])
  outputs[i,7:10] <- quantile(q$gamma[,2],c(.025,.975,.1,.9))
}
for (i in 1:11) {
  q <- extract.samples(ph2_reward_gradient_output[[i]])
  v <- inv_logit(q$gamma[,1] + q$gamma[,2]*(-2.5)*(6/21) + q$gamma[,2]*(-1.5)*(5/21) + q$gamma[,2]*(-0.5)*(4/21) + q$gamma[,2]*(0.5)*(3/21) + q$gamma[,2]*(1.5)*(2/21) + q$gamma[,2]*(2.5)*(1/21) + q$gamma[,3]*(-2.5)*(1/21) + q$gamma[,3]*(-1.5)*(2/21) + q$gamma[,3]*(-0.5)*(3/21) + q$gamma[,3]*(0.5)*(4/21) + q$gamma[,3]*(1.5)*(5/21) + q$gamma[,3]*(2.5)*(6/21))
  outputs[i+5,1] <- mean(v)
  outputs[i+5,2:5] <- quantile(v,c(.025,.975,.1,.9))
  outputs[i+5,6] <- mean(q$gamma[,2])
  outputs[i+5,7:10] <- quantile(q$gamma[,2],c(.025,.975,.1,.9))
  outputs[i+5,11] <- mean(q$gamma[,3])
  outputs[i+5,12:15] <- quantile(q$gamma[,3],c(.025,.975,.1,.9))
}
for (i in 1:9) {
  q <- extract.samples(ph3_all_pair_output[[i]])
  v <- inv_logit(q$gamma[,1] + q$gamma[,2]*(-2.5)*(6/21) + q$gamma[,2]*(-1.5)*(5/21) + q$gamma[,2]*(-0.5)*(4/21) + q$gamma[,2]*(0.5)*(3/21) + q$gamma[,2]*(1.5)*(2/21) + q$gamma[,2]*(2.5)*(1/21))
  outputs[i+16,1] <- mean(v)
  outputs[i+16,2:5] <- quantile(v,c(.025,.975,.1,.9))
  outputs[i+16,6] <- mean(q$gamma[,2])
  outputs[i+16,7:10] <- quantile(q$gamma[,2],c(.025,.975,.1,.9))
}
for (i in 1:9) {
  q <- extract.samples(ph4_reward_gradient_output[[i]])
  v <- inv_logit(q$gamma[,1] + q$gamma[,2]*(-2.5)*(6/21) + q$gamma[,2]*(-1.5)*(5/21) + q$gamma[,2]*(-0.5)*(4/21) + q$gamma[,2]*(0.5)*(3/21) + q$gamma[,2]*(1.5)*(2/21) + q$gamma[,2]*(2.5)*(1/21) + q$gamma[,3]*(-2.5)*(1/21) + q$gamma[,3]*(-1.5)*(2/21) + q$gamma[,3]*(-0.5)*(3/21) + q$gamma[,3]*(0.5)*(4/21) + q$gamma[,3]*(1.5)*(5/21) + q$gamma[,3]*(2.5)*(6/21))
  outputs[i+25,1] <- mean(v)
  outputs[i+25,2:5] <- quantile(v,c(.025,.975,.1,.9))
  outputs[i+25,6] <- mean(q$gamma[,2])
  outputs[i+25,7:10] <- quantile(q$gamma[,2],c(.025,.975,.1,.9))
  outputs[i+25,11] <- mean(q$gamma[,3])
  outputs[i+25,12:15] <- quantile(q$gamma[,3],c(.025,.975,.1,.9))
}

clip <- pipe("pbcopy", "w")                       
write.table(outputs, file=clip,sep="\t")                               
close(clip)

outputs <- matrix(0,nrow=20,ncol=15)
outputs[1,1] <- mean(exp1_ph1_qmean$gamma[,1])
outputs[1,2:5] <- quantile(exp1_ph1_qmean$gamma[,1],c(.005,.995,.1,.9))
outputs[1,6] <- mean(exp1_ph1_qmean$gamma[,2])
outputs[1,7:10] <- quantile(exp1_ph1_qmean$gamma[,2],c(.005,.995,.1,.9))

outputs[6,1] <- mean(exp1_ph2_qmean$gamma[,1])
outputs[6,2:5] <- quantile(exp1_ph2_qmean$gamma[,1],c(.005,.995,.1,.9))
outputs[6,6] <- mean(exp1_ph2_qmean$gamma[,2])
outputs[6,7:10] <- quantile(exp1_ph2_qmean$gamma[,2],c(.005,.995,.1,.9))
outputs[6,11] <- mean(exp1_ph2_qmean$gamma[,3])
outputs[6,12:15] <- quantile(exp1_ph2_qmean$gamma[,3],c(.005,.995,.1,.9))

outputs[11,1] <- mean(exp1_ph3_qmean$gamma[,1])
outputs[11,2:5] <- quantile(exp1_ph3_qmean$gamma[,1],c(.005,.995,.1,.9))
outputs[11,6] <- mean(exp1_ph3_qmean$gamma[,2])
outputs[11,7:10] <- quantile(exp1_ph3_qmean$gamma[,2],c(.005,.995,.1,.9))

outputs[16,1] <- mean(exp1_ph4_qmean$gamma[,1])
outputs[16,2:5] <- quantile(exp1_ph4_qmean$gamma[,1],c(.005,.995,.1,.9))
outputs[16,6] <- mean(exp1_ph4_qmean$gamma[,2])
outputs[16,7:10] <- quantile(exp1_ph4_qmean$gamma[,2],c(.005,.995,.1,.9))
outputs[16,11] <- mean(exp1_ph4_qmean$gamma[,3])
outputs[16,12:15] <- quantile(exp1_ph4_qmean$gamma[,3],c(.005,.995,.1,.9))

for (i in 1:4) {
	outputs[1+i,1] <- mean(exp1_ph1_qmean$beta[,i,1])
	outputs[1+i,2:5] <- quantile(exp1_ph1_qmean$beta[,i,1],c(.005,.995,.1,.9))
	outputs[1+i,6] <- mean(exp1_ph1_qmean$beta[,i,2])
	outputs[1+i,7:10] <- quantile(exp1_ph1_qmean$beta[,i,2],c(.005,.995,.1,.9))
	
	outputs[6+i,1] <- mean(exp1_ph2_qmean$beta[,i,1])
	outputs[6+i,2:5] <- quantile(exp1_ph2_qmean$beta[,i,1],c(.005,.995,.1,.9))
	outputs[6+i,6] <- mean(exp1_ph2_qmean$beta[,i,2])
	outputs[6+i,7:10] <- quantile(exp1_ph2_qmean$beta[,i,2],c(.005,.995,.1,.9))
	outputs[6+i,11] <- mean(exp1_ph2_qmean$beta[,i,3])
	outputs[6+i,12:15] <- quantile(exp1_ph2_qmean$beta[,i,3],c(.005,.995,.1,.9))
	
	outputs[11+i,1] <- mean(exp1_ph3_qmean$beta[,i,1])
	outputs[11+i,2:5] <- quantile(exp1_ph3_qmean$beta[,i,1],c(.005,.995,.1,.9))
	outputs[11+i,6] <- mean(exp1_ph3_qmean$beta[,i,2])
	outputs[11+i,7:10] <- quantile(exp1_ph3_qmean$beta[,i,2],c(.005,.995,.1,.9))
	
	outputs[16+i,1] <- mean(exp1_ph4_qmean$beta[,i,1])
	outputs[16+i,2:5] <- quantile(exp1_ph4_qmean$beta[,i,1],c(.005,.995,.1,.9))
	outputs[16+i,6] <- mean(exp1_ph4_qmean$beta[,i,2])
	outputs[16+i,7:10] <- quantile(exp1_ph4_qmean$beta[,i,2],c(.005,.995,.1,.9))
	outputs[16+i,11] <- mean(exp1_ph4_qmean$beta[,i,3])
	outputs[16+i,12:15] <- quantile(exp1_ph4_qmean$beta[,i,3],c(.005,.995,.1,.9))
	
}

clip <- pipe("pbcopy", "w")                       
write.table(outputs, file=clip,sep="\t")                               
close(clip)

outputs <- matrix(0,nrow=24,ncol=21)
d <- c(rep(-2.5,6),rep(-1.5,5),rep(-0.5,4),rep(0.5,3),rep(1.5,2),rep(2.5,1))
r <- c(-2.5,-1.5,-0.5,0.5,1.5,2.5,-2.5,-1.5,-0.5,0.5,1.5,-2.5,-1.5,-0.5,0.5,-2.5,-1.5,-0.5,-2.5,-1.5,-2.5)

for (i in 1:21) {
	outputs[1,i] <- mean(inv_logit(exp1_ph1_qmean$gamma[,1] + d[i]*exp1_ph1_qmean$gamma[,2]))
	outputs[2:5,i] <- quantile(inv_logit(exp1_ph1_qmean$gamma[,1] + d[i]*exp1_ph1_qmean$gamma[,2]),c(.005,.995,.1,.9))
	
	outputs[6,i] <- mean(inv_logit(exp1_ph2_qmean$gamma[,1] + d[i]*exp1_ph2_qmean$gamma[,2] + r[i]*exp1_ph2_qmean$gamma[,3]))
	outputs[7:10,i] <- quantile(inv_logit(exp1_ph2_qmean$gamma[,1] + d[i]*exp1_ph2_qmean$gamma[,2] + r[i]*exp1_ph2_qmean$gamma[,3]),c(.005,.995,.1,.9))
	
	outputs[11,i] <- mean(inv_logit(exp1_ph3_qmean$gamma[,1] + d[i]*exp1_ph3_qmean$gamma[,2]))
	outputs[12:15,i] <- quantile(inv_logit(exp1_ph3_qmean$gamma[,1] + d[i]*exp1_ph3_qmean$gamma[,2]),c(.005,.995,.1,.9))
	
	outputs[16,i] <- mean(inv_logit(exp1_ph4_qmean$gamma[,1] + d[i]*exp1_ph4_qmean$gamma[,2] + r[i]*exp1_ph4_qmean$gamma[,3]))
	outputs[17:20,i] <- quantile(inv_logit(exp1_ph4_qmean$gamma[,1] + d[i]*exp1_ph4_qmean$gamma[,2] + r[i]*exp1_ph4_qmean$gamma[,3]),c(.005,.995,.1,.9))
}

for (i in 1:21) {
	for (j in 1:4) {
		q <- rep(0,4)
		for (k in 1:4) {
			dex <- (RGMD1$Phase==j)&(RGMD1$PairIndex==i)&(RGMD1$Subject==k)
			q[k] <- sum(RGMD1$Successes[dex])/sum(RGMD1$Attempts[dex])
		}
		outputs[20+j,i] <- mean(q)
	}
}

clip <- pipe("pbcopy", "w")                       
write.table(outputs, file=clip,sep="\t")                               
close(clip)

#=======================================================================================
#====Experiment 2 - Reverse Gradient vs. Concordant Gradient - Adj-All Pair Transfer====
#=======================================================================================

exp2_sess_dex <- cbind(c(1,10,13,20,23,30,33,39,42,50,52,59,62,69,72,78,81,88,91,98,100,107,110,117),c(9,12,19,22,29,32,38,41,49,51,58,61,68,71,77,80,87,90,97,99,106,109,116,119),c(9,3,7,3,7,3,6,3,8,2,7,3,7,3,6,3,7,3,7,2,7,3,7,3),c(3,2,3,2,3,2,3,2,3,2,3,2,3,2,3,2,3,2,3,2,3,2,3,2),c(1,2,1,2,1,2,1,2,3,4,3,4,3,4,3,4,3,4,3,4,1,2,1,2))
exp2_output <- list()
exp2_dists <- list()

for (ph in 1:24) {
	x <- (exp2_sess_dex[ph,1]:exp2_sess_dex[ph,2])
	for (i in 1:exp2_sess_dex[ph,3]) {
		dex <- (RGMD2$Phase==ph)&(RGMD2$Session==i)
		dat1 <- list(N=sum(dex), S=length(unique(RGMD2$Subject[dex])), sub=RGMD2$Subject[dex], dist=RGMD2$Distance[dex]-3.5, reward=RGMD2$Reward[dex]-3.5, success=RGMD2$Successes[dex], attempt=RGMD2$Attempts[dex])
		exp2_output[x[i]] <- list(sampling(machines[[exp2_sess_dex[ph,4]]], data=dat1, iter=4000, warmup=2000, chains=4, cores=4, control=list(adapt_delta=0.99,max_treedepth=15)))
		precis(exp2_output[[x[i]]],pars="gamma",depth=2)
		dat2 <- list(N=sum(dex), S=length(unique(RGMD2$Subject[dex])), sub=RGMD2$Subject[dex], pair=RGMD2$PairIndex[dex], success=RGMD2$Successes[dex], attempt=RGMD2$Attempts[dex])
		exp2_dists[x[i]] <- list(sampling(machines[[4]], data=dat2, iter=4000, warmup=2000, chains=4, cores=4, control=list(adapt_delta=0.99,max_treedepth=15)))
		precis(exp2_dists[[x[i]]],pars=c("gamma","bigsigma","bigbeta1","bigbeta2"),depth=2)
	}
}

#====Training Performance By Session====

exp2_0_reverse_training_output <- list()
for (i in 1:6) {
	dex <- ((RGMD2$Phase==1)|(RGMD2$Phase==3)|(RGMD2$Phase==5)|(RGMD2$Phase==7)|(RGMD2$Phase==21)|(RGMD2$Phase==23))&(RGMD2$Session==i)
	dat <- list(N=sum(dex), S=length(unique(RGMD2$Subject[dex])), sub=RGMD2$Subject[dex], reward=RGMD2$Reward[dex]-3.5, success=RGMD2$Successes[dex], attempt=RGMD2$Attempts[dex])
	exp2_0_reverse_training_output[i] <- list(sampling(reward_gradient_adjacent_machine, data=dat, iter=4000, warmup=2000, chains=4, cores=4, control=list(adapt_delta=0.99,max_treedepth=15)))
	precis(exp2_0_reverse_training_output[[i]],pars="gamma",depth=2)
}

save.image("~/Projects/Academia/1 - Preprint/Reward Gradient TI (Jensen & Co.)/Stan Analysis/RGMD-StanOutput.RData")

exp2_0_concordant_training_output <- list()
for (i in 1:6) {
	dex <- ((RGMD2$Phase==9)|(RGMD2$Phase==11)|(RGMD2$Phase==13)|(RGMD2$Phase==15)|(RGMD2$Phase==17)|(RGMD2$Phase==19))&(RGMD2$Session==i)
	dat <- list(N=sum(dex), S=length(unique(RGMD2$Subject[dex])), sub=RGMD2$Subject[dex], reward=RGMD2$Reward[dex]-3.5, success=RGMD2$Successes[dex], attempt=RGMD2$Attempts[dex])
	exp2_0_concordant_training_output[i] <- list(sampling(reward_gradient_adjacent_machine, data=dat, iter=4000, warmup=2000, chains=4, cores=4, control=list(adapt_delta=0.99,max_treedepth=15)))
	precis(exp2_0_concordant_training_output[[i]],pars="gamma",depth=2)
}

save.image("~/Projects/Academia/1 - Preprint/Reward Gradient TI (Jensen & Co.)/Stan Analysis/RGMD-StanOutput.RData")

#====Test Performance By Session====

exp2_1_reverse_test_output <- list()
for (i in 1:2) {
	dex <- ((RGMD2$Phase==2)|(RGMD2$Phase==4)|(RGMD2$Phase==6)|(RGMD2$Phase==8)|(RGMD2$Phase==22)|(RGMD2$Phase==24))&(RGMD2$Session==i)
	dat <- list(N=sum(dex), S=length(unique(RGMD2$Subject[dex])), sub=RGMD2$Subject[dex], dist=RGMD2$Distance[dex]-3.5, reward=RGMD2$Reward[dex]-3.5, success=RGMD2$Successes[dex], attempt=RGMD2$Attempts[dex])
	exp2_1_reverse_test_output[i] <- list(sampling(reward_gradient_machine, data=dat, iter=4000, warmup=2000, chains=4, cores=4, control=list(adapt_delta=0.99,max_treedepth=15)))
	precis(exp2_1_reverse_test_output[[i]],pars="gamma",depth=2)
}

exp2_1_concordant_test_output <- list()
for (i in 1:2) {
	dex <- ((RGMD2$Phase==10)|(RGMD2$Phase==12)|(RGMD2$Phase==14)|(RGMD2$Phase==16)|(RGMD2$Phase==18)|(RGMD2$Phase==20))&(RGMD2$Session==i)
	dat <- list(N=sum(dex), S=length(unique(RGMD2$Subject[dex])), sub=RGMD2$Subject[dex], dist=RGMD2$Distance[dex]-3.5, reward=RGMD2$Reward[dex]-3.5, success=RGMD2$Successes[dex], attempt=RGMD2$Attempts[dex])
	exp2_1_concordant_test_output[i] <- list(sampling(reward_gradient_machine, data=dat, iter=4000, warmup=2000, chains=4, cores=4, control=list(adapt_delta=0.99,max_treedepth=15)))
	precis(exp2_1_concordant_test_output[[i]],pars="gamma",depth=2)
}

save.image("~/Projects/Academia/1 - Preprint/Reward Gradient TI (Jensen & Co.)/Stan Analysis/RGMD-StanOutput.RData")

#====Training Representation By Session====

exp2_0_reverse_training_dists <- list()
for (i in 1:6) {
	print(c("reverse training",i))
	dex <- ((RGMD2$Phase==1)|(RGMD2$Phase==3)|(RGMD2$Phase==5)|(RGMD2$Phase==7)|(RGMD2$Phase==21)|(RGMD2$Phase==23))&(RGMD2$Session==i)
	dat <- list(N=sum(dex), S=length(unique(RGMD2$Subject[dex])), sub=RGMD2$Subject[dex], pair=RGMD2$PairIndex[dex], success=RGMD2$Successes[dex], attempt=RGMD2$Attempts[dex])
	exp2_0_reverse_training_dists[i] <- list(sampling(machines[[4]], data=dat, iter=4000, warmup=2000, chains=4, cores=4, control=list(adapt_delta=0.99,max_treedepth=17)))
}
save.image("~/Projects/Academia/1 - Preprint/Reward Gradient TI (Jensen & Co.)/Stan Analysis/RGMD-StanOutput.RData")

exp2_0_concordant_training_dists <- list()
for (i in 1:6) {
	print(c("concordant training",i))
	dex <- ((RGMD2$Phase==9)|(RGMD2$Phase==11)|(RGMD2$Phase==13)|(RGMD2$Phase==15)|(RGMD2$Phase==17)|(RGMD2$Phase==19))&(RGMD2$Session==i)
	dat <- list(N=sum(dex), S=length(unique(RGMD2$Subject[dex])), sub=RGMD2$Subject[dex], pair=RGMD2$PairIndex[dex], success=RGMD2$Successes[dex], attempt=RGMD2$Attempts[dex])
	exp2_0_concordant_training_dists[i] <- list(sampling(machines[[4]], data=dat, iter=4000, warmup=2000, chains=4, cores=4, control=list(adapt_delta=0.99,max_treedepth=17)))
}
save.image("~/Projects/Academia/1 - Preprint/Reward Gradient TI (Jensen & Co.)/Stan Analysis/RGMD-StanOutput.RData")

#====Test Representation By Session====

exp2_1_reverse_test_dists <- list()
for (i in 1:2) {
	print(c("reverse testing",i))
	dex <- ((RGMD2$Phase==2)|(RGMD2$Phase==4)|(RGMD2$Phase==6)|(RGMD2$Phase==8)|(RGMD2$Phase==22)|(RGMD2$Phase==24))&(RGMD2$Session==i)
	dat <- list(N=sum(dex), S=length(unique(RGMD2$Subject[dex])), sub=RGMD2$Subject[dex], pair=RGMD2$PairIndex[dex], success=RGMD2$Successes[dex], attempt=RGMD2$Attempts[dex])
	exp2_1_reverse_test_dists[i] <- list(sampling(machines[[4]], data=dat, iter=4000, warmup=2000, chains=4, cores=4, control=list(adapt_delta=0.99,max_treedepth=17)))
}

exp2_1_concordant_test_dists <- list()
for (i in 1:2) {
	print(c("concordant testing",i))
	dex <- ((RGMD2$Phase==10)|(RGMD2$Phase==12)|(RGMD2$Phase==14)|(RGMD2$Phase==16)|(RGMD2$Phase==18)|(RGMD2$Phase==20))&(RGMD2$Session==i)
	dat <- list(N=sum(dex), S=length(unique(RGMD2$Subject[dex])), sub=RGMD2$Subject[dex], pair=RGMD2$PairIndex[dex], success=RGMD2$Successes[dex], attempt=RGMD2$Attempts[dex])
	exp2_1_concordant_test_dists[i] <- list(sampling(machines[[4]], data=dat, iter=4000, warmup=2000, chains=4, cores=4, control=list(adapt_delta=0.99,max_treedepth=17)))
}
save.image("~/Projects/Academia/1 - Preprint/Reward Gradient TI (Jensen & Co.)/Stan Analysis/RGMD-StanOutput.RData")

#====Q-learning By Phase: Subject Best Fits====

exp2_qlearning <- list()
initL <- list(list(z=c(0,0)),list(z=c(0,0)),list(z=c(0,0)),list(z=c(0,0)))
for (s in 1:4) {
	print(c("subject",s))
	dex <- (RGMDL2$Subject==s)
	dat <- list(N=sum(dex), C=7, reset=(RGMDL2$Phase[dex]%%2)*(RGMDL2$Trial[dex]==1)*(RGMDL2$Session[dex]==1), reward=RGMDL2$Response[dex]*RGMDL2$Reward[dex], choice=RGMDL2$Target[dex], altern=RGMDL2$Distractor[dex])
	for (t in 1:dat$N) {
		if (dat$reward[t]==0) {
			tmp <- 	dat$choice[t]
			dat$choice[t] <- dat$altern[t]
			dat$altern[t] <- tmp
		}
	}
	exp2_qlearning[s] <- list(sampling(q_learning_machine, data=dat, iter=4000, warmup=2000, chains=4, cores=4, init=initL, control=list(adapt_delta=0.96,max_treedepth=12)))
}
save.image("~/Projects/Academia/1 - Preprint/Reward Gradient TI (Jensen & Co.)/Stan Analysis/RGMD-StanOutput.RData")

tr_stims <- cbind(c(1,2,3,4,5,6,2,3,4,5,6,7),c(2,3,4,5,6,7,1,2,3,4,5,6))
te_stims <- cbind(c(1,2,3,4,5,6,1,2,3,4,5,1,2,3,4,1,2,3,1,2,1,2,3,4,5,6,7,3,4,5,6,7,4,5,6,7,5,6,7,6,7,7),c(2,3,4,5,6,7,3,4,5,6,7,4,5,6,7,5,6,7,6,7,7,1,2,3,4,5,6,1,2,3,4,5,1,2,3,4,1,2,3,1,2,1))
rewardls <- list(c(1,2,3,4,5,6,1,2,3,4,5,6),c(1,2,3,4,5,6,1,2,3,4,5,1,2,3,4,1,2,3,1,2,1,1,2,3,4,5,6,1,2,3,4,5,1,2,3,4,1,2,3,1,2,1),c(6,5,4,3,2,1,6,5,4,3,2,1),c(6,5,4,3,2,1,6,5,4,3,2,6,5,4,3,6,5,4,6,5,6,6,5,4,3,2,1,6,5,4,3,2,6,5,4,3,6,5,4,6,5,6))

sim_output <- array(0,dim=c(8000,119,4))
for (sub in 1:4) {
	pl <- extract.samples(exp2_qlearning[[s]])
	print(c("subject",sub))
	for (par in 1:8000) {
		if (par%%1000==0) {
			print(c("sample",par))
		}
		for (ph in 1:24) {
			if (ph%%2==1) {
				mem <- rep(5,7)
				stims <- tr_stims
			} else {
				stims <- te_stims
			}
			for (sess in exp2_sess_dex[ph,1]:exp2_sess_dex[ph,2]) {
				q <- c()
				r <- 0
				rand <- runif(600,0,1)
				for (t in 1:600) {
					if (length(q)==0) {
						if (ph%%2==1) {
							q <- sample(1:12,12)
						} else {
							q <- sample(1:42,42)
						}
					}
					tl <- sort(stims[q[1],])
					rw <- rewardls[[exp2_sess_dex[ph,5]]][q[1]]
					q <- q[-1]
					p <- exp(mem[tl[1]]^pl$beta[par])/(exp(mem[tl[1]]^pl$beta[par])+exp(mem[tl[2]]^pl$beta[par]))
					if (rand[t] < p) {
						mem[tl[1]] = (1-pl$alpha[par])*mem[tl[1]] + pl$alpha[par]*rw
						r <- r+1
					} else {
						mem[tl[2]] = (1-pl$alpha[par])*mem[tl[2]]
					}
				}
				sim_output[par,sess,sub] <- r/600
			}
		}
	}
}

#====Q-learning By Phase: Optimized (alpha = 0.15, beta = 3)====

sim_opt_output <- rep(0,119)
for (cycle in 1:800) {
	for (ph in 1:24) {
		if (ph%%2==1) {
			mem <- rep(5,7)
			stims <- tr_stims
		} else {
			stims <- te_stims
		}
		for (sess in exp2_sess_dex[ph,1]:exp2_sess_dex[ph,2]) {
			q <- c()
			r <- 0
			rand <- runif(600,0,1)
			for (t in 1:600) {
				if (length(q)==0) {
					if (ph%%2==1) {
						q <- sample(1:12,12)
					} else {
						q <- sample(1:42,42)
					}
				}
				tl <- sort(stims[q[1],])
				rw <- rewardls[[exp2_sess_dex[ph,5]]][q[1]]
				q <- q[-1]
				p <- exp(mem[tl[1]]^3)/(exp(mem[tl[1]]^3)+exp(mem[tl[2]]^3))
				if (rand[t] < p) {
					mem[tl[1]] = (1-0.15)*mem[tl[1]] + 0.15*rw
					r <- r+1
				} else {
					mem[tl[2]] = (1-0.15)*mem[tl[2]]
				}
			}
			sim_opt_output[sess] <- sim_opt_output[sess]+r
		}
	}
}
sim_opt_output <- sim_opt_output/(600*800)

#====Representation Content Analysis====

h0 <- c(2,1,1,1,1,1,0)
h1 <- c(6,5,4,3,2,1,0)
h2 <- c(1,1,1.5,2,2.5,3,0)
h3 <- c(6,10,12,12,10,6,0)
h4 <- c(12,10,4,3,2,1,0)
h5 <- c(36,25,16,9,4,1,0)

q <- 0
for (a in 1:6) {
	for (b in a:7) {
		if (h3[a]>h3[b]) {
			q <- q+1
		} else if (h3[a]==h3[b]) {
			q <- q+0.5
		}
	}
}

exp2_rep_series <- matrix(0,135,7)
exp2_pair_series <- matrix(0,135,3)
exp2_bf_series <- rep(0,135)

for (ph in 1:24) {
	for (sess in exp2_sess_dex[ph,1]:exp2_sess_dex[ph,2]){
		q <- extract.samples(exp2_dists[[sess]])
		ps <- rep(0,8000)
		for (j in 1:8000) {
			m <- mean(q$gamma[j,])
			s <- sd(q$gamma[j,])
			q$gamma[j,] <- (q$gamma[j,] - m)/s
			q$bigsigma[j,] <- q$bigsigma[j,]/s
			for (a in 1:6) {
				for (b in a:7) {
					if (q$gamma[j,a]>q$gamma[j,b]) {
						ps[j] <- ps[j]+1
					}
				}
			}
		}
		exp2_pair_series[sess,1] <- mean(ps)
		psu <- ps/21
		m1 <- mean(psu)
		m2 <- var(psu)
		ahat <- m1*(((m1*(1-m1))/m2)-1)
		bhat <- (1-m1)*(((m1*(1-m1))/m2)-1)
		exp2_pair_series[sess,2:3] <- qbeta(c(.025,.975),ahat,bhat)*21
		
		exp2_rep_series[sess,] <- apply(q$gamma,2,mean)
		
		LBIC <- rep(0,8000)
		RBIC <- rep(0,8000)
		for (j in 1:8000) {
			d <- q$gamma[j,]
			d <- cbind(d,h1)
			if (exp2_sess_dex[ph,5] == 1) {
				d <- cbind(d,h2)
			} else if (exp2_sess_dex[ph,5] == 2) {
				d <- cbind(d,h3)
			} else if (exp2_sess_dex[ph,5] == 3) {
				d <- cbind(d,h4)
			} else if (exp2_sess_dex[ph,5] == 4) {
				d <- cbind(d,h5)
			}
			linm <- lm(d[,1] ~ d[,2])
			rewm <- lm(d[,1] ~ d[,3])
			LBIC[j] <- BIC(linm)
			RBIC[j] <- BIC(rewm)
		}
		exp2_bf_series[sess] <- mean(exp(-(LBIC))/(exp(-(LBIC))+exp(-(RBIC))))
	}
}

